@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -266,446 +266,11 @@ static void send_message(xdebug_con *context, xdebug_xml_node *message TSRMLS_DC
 	xdebug_str_ptr_dtor(tmp);
 }
 
-
-/*****************************************************************************
-** Data returning functions
-*/
-#define XF_ST_ROOT               0
-#define XF_ST_ARRAY_INDEX_NUM    1
-#define XF_ST_ARRAY_INDEX_ASSOC  2
-#define XF_ST_OBJ_PROPERTY       3
-#define XF_ST_STATIC_ROOT        4
-#define XF_ST_STATIC_PROPERTY    5
-
-inline static HashTable *fetch_ht_from_zval(zval *z TSRMLS_DC)
-{
-	switch (Z_TYPE_P(z)) {
-		case IS_ARRAY:
-			return Z_ARRVAL_P(z);
-			break;
-		case IS_OBJECT:
-			return Z_OBJPROP_P(z);
-			break;
-	}
-	return NULL;
-}
-
-inline static char *fetch_classname_from_zval(zval *z, int *length, zend_class_entry **ce TSRMLS_DC)
-{
-	char *name;
-	zend_uint name_len;
-	zend_class_entry *tmp_ce;
-
-	if (Z_TYPE_P(z) != IS_OBJECT) {
-		return NULL;
-	}
-
-	tmp_ce = zend_get_class_entry(z TSRMLS_CC);
-	if (Z_OBJ_HT_P(z)->get_class_name == NULL ||
-		Z_OBJ_HT_P(z)->get_class_name(z, (const char **) &name, &name_len, 0 TSRMLS_CC) != SUCCESS) {
-
-		if (!tmp_ce) {
-			return NULL;
-		}
-
-		*length = tmp_ce->name_length;
-		*ce = tmp_ce;
-		return estrdup(tmp_ce->name);
-	} else {
-		*ce = tmp_ce;
-	}
-
-	*length = name_len;
-	return name;
-}
-
-static char* prepare_search_key(char *name, int *name_length, char *prefix, int prefix_length)
-{
-	char *element;
-	int   extra_length = 0;
-
-	if (prefix_length) {
-		if (prefix[0] == '*') {
-			extra_length = 3;
-		} else {
-			extra_length = 2 + prefix_length;
-		}
-	}
-
-	element = malloc(*name_length + 1 + extra_length);
-	memset(element, 0, *name_length + 1 + extra_length);
-	if (extra_length) {
-		memcpy(element + 1, prefix, extra_length - 2);
-	}
-	memcpy(element + extra_length, name, *name_length);
-	*name_length += extra_length;
-
-	return element;
-}
-
-static zval* fetch_zval_from_symbol_table(HashTable *ht, char* name, int name_length, int type, char* ccn, int ccnl, zend_class_entry *cce TSRMLS_DC)
-{
-	zval **retval_pp = NULL, *retval_p = NULL;
-	char  *element = NULL;
-	int    element_length = name_length;
-#if PHP_VERSION_ID >= 50400
-	zend_property_info *zpp;
-#endif
-
-	switch (type) {
-		case XF_ST_STATIC_ROOT:
-		case XF_ST_STATIC_PROPERTY:
-#if PHP_VERSION_ID < 50400
-			ht = CE_STATIC_MEMBERS(cce);
-			goto continue_from_static_root;
-#else
-			/* First we try a public,private,protected property */
-			element = prepare_search_key(name, &element_length, "", 0);
-			if (cce && &cce->properties_info && zend_hash_find(&cce->properties_info, element, element_length + 1, (void **) &zpp) == SUCCESS) {
-				retval_p = cce->static_members_table[zpp->offset];
-				goto cleanup;
-			}
-			element_length = name_length;
-
-			/* Then we try to see whether the first char is * and use the part between * and * as class name for the private property */
-			if (name[0] == '*') {
-				char *secondStar;
-				
-				secondStar = strstr(name + 1, "*");
-				if (secondStar) {
-					free(element);
-					element_length = name_length - (secondStar + 1 - name);
-					element = prepare_search_key(secondStar + 1, &element_length, "", 0);
-					if (cce && &cce->properties_info && zend_hash_find(&cce->properties_info, element, element_length + 1, (void **) &zpp) == SUCCESS) {
-						retval_p = cce->static_members_table[zpp->offset];
-						goto cleanup;
-					}
-				}
-			}
-
-			break;
-#endif
-
-		case XF_ST_ROOT:
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
-			/* Check for compiled vars */
-			element = prepare_search_key(name, &element_length, "", 0);
-			if (XG(active_execute_data) && XG(active_op_array)) {
-				int i = 0;
-				ulong hash_value = zend_inline_hash_func(element, element_length + 1);
-				zend_op_array *opa = XG(active_op_array);
-				zval **CV;
-
-				while (i < opa->last_var) {
-					if (opa->vars[i].hash_value == hash_value &&
-						opa->vars[i].name_len == element_length &&
-						strcmp(opa->vars[i].name, element) == 0)
-					{
-#if PHP_VERSION_ID >= 50500
-						CV = (*EX_CV_NUM(XG(active_execute_data), i));
-#else
-						CV = XG(active_execute_data)->CVs[i];
-#endif
-						if (CV) {
-							retval_p = *CV;
-							goto cleanup;
-						}
-					}
-					i++;
-				}
-			}
-			free(element);
-			ht = XG(active_symbol_table);
-#else
-			ht = XG(active_symbol_table);
-			/* break intentionally missing */
-#endif
-		case XF_ST_ARRAY_INDEX_ASSOC:
-			element = prepare_search_key(name, &name_length, "", 0);
-
-			/* Handle "this" in a different way */
-			if (type == XF_ST_ROOT && strcmp("this", element) == 0) {
-				if (XG(This)) {
-					retval_p = XG(This);
-				} else {
-					retval_p = NULL;
-				}
-				goto cleanup;
-			}
-
-			if (ht && zend_hash_find(ht, element, name_length + 1, (void **) &retval_pp) == SUCCESS) {
-				retval_p = *retval_pp;
-				goto cleanup;
-			}
-			break;
-		case XF_ST_ARRAY_INDEX_NUM:
-			element = prepare_search_key(name, &name_length, "", 0);
-			if (ht && zend_hash_index_find(ht, strtoul(element, NULL, 10), (void **) &retval_pp) == SUCCESS) {
-				retval_p = *retval_pp;
-				goto cleanup;
-			}
-			break;
-		case XF_ST_OBJ_PROPERTY:
-#if PHP_VERSION_ID <= 50400
-continue_from_static_root:
-#endif
-
-			/* First we try a public property */
-			element = prepare_search_key(name, &element_length, "", 0);
-			if (ht && zend_symtable_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
-				retval_p = *retval_pp;
-				goto cleanup;
-			}
-			element_length = name_length;
-
-			/* Then we try it again as protected property */
-			free(element);
-			element = prepare_search_key(name, &element_length, "*", 1);
-			if (ht && zend_hash_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
-				retval_p = *retval_pp;
-				goto cleanup;
-			}
-			element_length = name_length;
-
-			/* Then we try it again as private property */
-			free(element);
-			element = prepare_search_key(name, &element_length, ccn, ccnl);
-			if (ht && zend_hash_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
-				retval_p = *retval_pp;
-				goto cleanup;
-			}
-			element_length = name_length;
-
-			/* Then we try to see whether the first char is * and use the part between * and * as class name for the private property */
-			if (name[0] == '*') {
-				char *secondStar;
-				
-				secondStar = strstr(name + 1, "*");
-				if (secondStar) {
-					free(element);
-					element_length = name_length - (secondStar + 1 - name);
-					element = prepare_search_key(secondStar + 1, &element_length, name + 1, secondStar - name - 1);
-					if (ht && zend_hash_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
-						retval_p = *retval_pp;
-						goto cleanup;
-					}
-				}
-			}
-
-			break;
-	}
-cleanup:
-	if (element) {
-		free(element);
-	}
-	return retval_p;
-}
-
-static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
-{
-	HashTable *st = NULL;
-	int        found = -1;
-	int        state = 0;
-	char     **p = &name;
-	char      *keyword = NULL, *keyword_end = NULL;
-	int        type = XF_ST_ROOT;
-	zval      *retval = NULL;
-	char      *current_classname = NULL;
-	zend_class_entry *current_ce = NULL;
-	int        cc_length = 0;
-	char       quotechar = 0;
-
-	do {
-		if (*p[0] == '\0') {
-			found = 0;
-		} else {
-			switch (state) {
-				case 0:
-					if (*p[0] == '$') {
-						keyword = *p + 1;
-						break;
-					}
-					if (*p[0] == ':') { /* special tricks */
-						keyword = *p;
-						state = 7;
-						break;
-					}
-					keyword = *p;
-					state = 1;
-					/* break intentionally missing */
-				case 1:
-					if (*p[0] == '[') {
-						keyword_end = *p;
-						if (keyword) {
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
-							if (current_classname) {
-								efree(current_classname);
-							}
-							current_classname = NULL;
-							cc_length = 0;
-							current_ce = NULL;
-							if (retval) {
-								st = fetch_ht_from_zval(retval TSRMLS_CC);
-							}
-							keyword = NULL;
-						}
-						state = 3;
-					} else if (*p[0] == '-') {
-						keyword_end = *p;
-						if (keyword) {
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
-							if (current_classname) {
-								efree(current_classname);
-							}
-							current_classname = NULL;
-							cc_length = 0;
-							current_ce = NULL;
-							if (retval) {
-								current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
-								st = fetch_ht_from_zval(retval TSRMLS_CC);
-							}
-							keyword = NULL;
-						}
-						state = 2;
-						type = XF_ST_OBJ_PROPERTY;
-					} else if (*p[0] == ':') {
-						keyword_end = *p;
-						if (keyword) {
-							retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
-							if (current_classname) {
-								efree(current_classname);
-							}
-							current_classname = NULL;
-							cc_length = 0;
-							if (retval) {
-								current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
-								st = NULL;
-							}
-							keyword = NULL;
-						}
-						state = 8;
-						type = XF_ST_STATIC_PROPERTY;
-					}
-					break;
-				case 2:
-					if (*p[0] != '>') {
-						keyword = *p;
-						state = 1;
-					}
-					break;
-				case 8:
-					if (*p[0] != ':') {
-						keyword = *p;
-						state = 1;
-					}
-					break;
-				case 3: /* Parsing in [...] */
-					/* Associative arrays */
-					if (*p[0] == '\'' || *p[0] == '"') {
-						state = 4;
-						keyword = *p + 1;
-						quotechar = *p[0];
-						type = XF_ST_ARRAY_INDEX_ASSOC;
-					}
-					/* Numerical index */
-					if (*p[0] >= '0' && *p[0] <= '9') {
-						cc_length = 0;
-						state = 6;
-						keyword = *p;
-						type = XF_ST_ARRAY_INDEX_NUM;
-					}
-					/* Numerical index starting with a - */
-					if (*p[0] == '-') {
-						state = 9;
-						keyword = *p;
-					}
-					break;
-				case 9:
-					/* Numerical index starting with a - */
-					if (*p[0] >= '0' && *p[0] <= '9') {
-						state = 6;
-						type = XF_ST_ARRAY_INDEX_NUM;
-					}
-					break;
-				case 4:
-					if (*p[0] == quotechar) {
-						quotechar = 0;
-						state = 5;
-						keyword_end = *p;
-						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
-						if (current_classname) {
-							efree(current_classname);
-						}
-						current_classname = NULL;
-						cc_length = 0;
-						if (retval) {
-							current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
-							st = fetch_ht_from_zval(retval TSRMLS_CC);
-						}
-						keyword = NULL;
-					}
-					break;
-				case 5:
-					if (*p[0] == ']') {
-						state = 1;
-					}
-					break;
-				case 6:
-					if (*p[0] == ']') {
-						state = 1;
-						keyword_end = *p;
-						retval = fetch_zval_from_symbol_table(st, keyword, keyword_end - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
-						if (current_classname) {
-							efree(current_classname);
-						}
-						current_classname = NULL;
-						cc_length = 0;
-						if (retval) {
-							current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
-							st = fetch_ht_from_zval(retval TSRMLS_CC);
-						}
-						keyword = NULL;
-					}
-					break;
-				case 7: /* special cases, started with a ":" */
-					if (*p[0] == ':') {
-						state = 1;
-						keyword_end = *p;
-
-						if (strncmp(keyword, "::", 2) == 0) { /* static class properties */
-							zend_class_entry *ce = zend_fetch_class(XG(active_fse)->function.class, strlen(XG(active_fse)->function.class), ZEND_FETCH_CLASS_SELF TSRMLS_CC);
-
-							st = NULL;
-
-							current_classname = estrdup(ce->name);
-							cc_length = strlen(ce->name);
-							current_ce = ce;
-							keyword = *p + 1;
-
-							type = XF_ST_STATIC_ROOT;
-						} else {
-							keyword = NULL;
-						}
-					}
-					break;
-			}
-			(*p)++;
-		}
-	} while (found < 0);
-	if (keyword != NULL) {
-		retval = fetch_zval_from_symbol_table(st, keyword, *p - keyword, type, current_classname, cc_length, current_ce TSRMLS_CC);
-	}
-	if (current_classname) {
-		efree(current_classname);
-	}
-	return retval;
-}
-
 static xdebug_xml_node* get_symbol(char* name, int name_length, xdebug_var_export_options *options TSRMLS_DC)
 {
 	zval                *retval;
 
-	retval = get_symbol_contents_zval(name, name_length TSRMLS_CC);
+	retval = xdebug_get_php_symbol(name, name_length TSRMLS_CC);
 	if (retval) {
 		return xdebug_get_zval_value_xml_node(name, retval, options TSRMLS_CC);
 	}
@@ -717,7 +282,7 @@ static int get_symbol_contents(char* name, int name_length, xdebug_xml_node *nod
 {
 	zval                *retval;
 
-	retval = get_symbol_contents_zval(name, name_length TSRMLS_CC);
+	retval = xdebug_get_php_symbol(name, name_length TSRMLS_CC);
 	if (retval) {
 		xdebug_var_export_xml_node(&retval, name, node, options, 1 TSRMLS_CC);
 		return 1;
@@ -1331,20 +896,10 @@ static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 	zend_op_array     *original_active_op_array = EG(active_op_array);
 	zend_execute_data *original_execute_data = EG(current_execute_data);
 	int                original_no_extensions = EG(no_extensions);
-#if PHP_VERSION_ID < 50200
-	zend_bool          original_bailout_set = EG(bailout_set);
-	jmp_buf            original_bailout;
-#else
+	zval              *original_exception = EG(exception);
 	jmp_buf           *original_bailout = EG(bailout);
-#endif
-#if PHP_VERSION_ID >= 50300
 	void             **original_argument_stack_top = EG(argument_stack)->top;
 	void             **original_argument_stack_end = EG(argument_stack)->end;
-#endif
-
-#if PHP_VERSION_ID < 50200
-	memcpy(&original_bailout, &EG(bailout), sizeof(jmp_buf));
-#endif
 
 	/* Remember error reporting level */
 	old_error_reporting = EG(error_reporting);
@@ -1353,6 +908,9 @@ static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 	/* Do evaluation */
 	XG(breakpoints_allowed) = 0;
 
+	/* Reset exception in case we're triggered while being in xdebug_throw_exception_hook */
+	EG(exception) = NULL;
+
 	zend_first_try {
 		res = zend_eval_string(eval_string, ret_zval, "xdebug://debug-eval" TSRMLS_CC);
 	} zend_end_try();
@@ -1366,16 +924,10 @@ static int xdebug_do_eval(char *eval_string, zval *ret_zval TSRMLS_DC)
 	EG(active_op_array) = original_active_op_array;
 	EG(current_execute_data) = original_execute_data;
 	EG(no_extensions) = original_no_extensions;
-#if PHP_VERSION_ID < 50200
-	EG(bailout_set) = original_bailout_set;
-	memcpy(&EG(bailout), &original_bailout, sizeof(jmp_buf));
-#else
+	EG(exception) = original_exception;
 	EG(bailout) = original_bailout;
-#endif
-#if PHP_VERSION_ID >= 50300
 	EG(argument_stack)->top = original_argument_stack_top;
 	EG(argument_stack)->end = original_argument_stack_end;
-#endif
 
 	return res;
 }
@@ -1723,6 +1275,19 @@ DBGP_FUNC(typemap_get)
 	}
 }
 
+static int add_constant_node(xdebug_xml_node *node, char *name, int name_lenth, zval *const_val, xdebug_var_export_options *options TSRMLS_DC)
+{
+	xdebug_xml_node *contents;
+
+	contents = xdebug_get_zval_value_xml_node_ex(name, const_val, XDEBUG_VAR_TYPE_CONSTANT, options TSRMLS_CC);
+	if (contents) {
+		xdebug_xml_add_attribute(contents, "facet", "constant");
+		xdebug_xml_add_child(node, contents);
+		return SUCCESS;
+	}
+	return FAILURE;
+}
+
 static int add_variable_node(xdebug_xml_node *node, char *name, int name_length, int var_only, int non_null, int no_eval, xdebug_var_export_options *options TSRMLS_DC)
 {
 	xdebug_xml_node *contents;
@@ -1759,7 +1324,6 @@ DBGP_FUNC(property_get)
 	/* Set the symbol table corresponding with the requested stack depth */
 	if (context_nr == 0) { /* locals */
 		if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 			function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
 
 			if (depth > 0) {
@@ -1767,18 +1331,18 @@ DBGP_FUNC(property_get)
 			} else {
 				XG(active_execute_data) = EG(current_execute_data);
 			}
-#else
-			XG(active_execute_data) = fse->execute_data;
-#endif
 			XG(active_symbol_table) = fse->symbol_table;
-			XG(active_op_array)     = fse->op_array;
 			XG(This)                = fse->This;
 			XG(active_fse)          = fse;
 		} else {
 			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_STACK_DEPTH_INVALID);
 		}
-	} else { /* superglobals */
+	} else if (context_nr == 1) { /* superglobals */
 		XG(active_symbol_table) = &EG(symbol_table);
+	} else if (context_nr == 2) { /* constants */
+		/* Do nothing */
+	} else {
+		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_INVALID_ARGS);
 	}
 
 	if (CMD_OPTION('p')) {
@@ -1792,12 +1356,34 @@ DBGP_FUNC(property_get)
 	if (CMD_OPTION('m')) {
 		options->max_data= strtol(CMD_OPTION('m'), NULL, 10);
 	}
-	if (add_variable_node(*retval, CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1, 1, 0, 0, options TSRMLS_CC) == FAILURE) {
-		options->max_data = old_max_data;
-		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTANT);
+	if (context_nr == 2) { /* constants */
+		zval const_val;
+
+		if (!zend_get_constant(CMD_OPTION('n'), strlen(CMD_OPTION('n')), &const_val TSRMLS_CC)) {
+			options->max_data = old_max_data;
+			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTENT);
+		}
+		if (add_constant_node(*retval, CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1, &const_val, options TSRMLS_CC) == FAILURE) {
+			options->max_data = old_max_data;
+			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTENT);
+		}
+	} else {
+		if (add_variable_node(*retval, CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1, 1, 0, 0, options TSRMLS_CC) == FAILURE) {
+			options->max_data = old_max_data;
+			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTENT);
+		}
 	}
 	options->max_data = old_max_data;
-	XG(active_op_array) = NULL;
+}
+
+static void set_vars_from_EG(TSRMLS_D)
+{
+	EG(opline_ptr) = &EG(current_execute_data)->opline;
+	EG(active_op_array) = EG(current_execute_data)->op_array;
+	EG(active_symbol_table) = EG(current_execute_data)->symbol_table;
+	EG(This) = EG(current_execute_data)->current_this;
+	EG(scope) = EG(current_execute_data)->current_scope;
+	EG(called_scope) = EG(current_execute_data)->current_called_scope;
 }
 
 DBGP_FUNC(property_set)
@@ -1813,6 +1399,7 @@ DBGP_FUNC(property_set)
 	function_stack_entry      *fse;
 	xdebug_var_export_options *options = (xdebug_var_export_options*) context->options;
 	zval                      *symbol;
+	zend_execute_data         *original_execute_data;
 	XDEBUG_STR_SWITCH_DECL;
 
 	if (!CMD_OPTION('n')) { /* name */
@@ -1834,7 +1421,6 @@ DBGP_FUNC(property_set)
 	/* Set the symbol table corresponding with the requested stack depth */
 	if (context_nr == 0) { /* locals */
 		if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 			function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
 
 			if (depth > 0) {
@@ -1842,11 +1428,7 @@ DBGP_FUNC(property_set)
 			} else {
 				XG(active_execute_data) = EG(current_execute_data);
 			}
-#else
-			XG(active_execute_data) = fse->execute_data;
-#endif
 			XG(active_symbol_table) = fse->symbol_table;
-			XG(active_op_array)     = fse->op_array;
 			XG(This)                = fse->This;
 			XG(active_fse)          = fse;
 		} else {
@@ -1865,12 +1447,12 @@ DBGP_FUNC(property_set)
 	new_value = (char*) xdebug_base64_decode((unsigned char*) data, strlen(data), &new_length);
 
 	if (CMD_OPTION('t')) {
-		symbol = get_symbol_contents_zval(CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1 TSRMLS_CC);
+		symbol = xdebug_get_php_symbol(CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1 TSRMLS_CC);
 
 		/* Handle result */
 		if (!symbol) {
 			efree(new_value);
-			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTANT);
+			RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTENT);
 		} else {
 			zval_dtor(symbol);
 			Z_TYPE_P(symbol) = IS_STRING;
@@ -1901,10 +1483,24 @@ DBGP_FUNC(property_set)
 			}
 		}
 	} else {
+		/* backup executor state */
+		if (depth > 0) {
+			original_execute_data = EG(current_execute_data);
+
+			EG(current_execute_data) = XG(active_execute_data);
+			set_vars_from_EG(TSRMLS_C);
+		}
+
 		/* Do the eval */
 		eval_string = xdebug_sprintf("%s = %s", CMD_OPTION('n'), new_value);
 		res = xdebug_do_eval(eval_string, &ret_zval TSRMLS_CC);
 
+		/* restore executor state */
+		if (depth > 0) {
+			EG(current_execute_data) = original_execute_data;
+			set_vars_from_EG(TSRMLS_C);
+		}
+
 		/* Free data */
 		xdfree(eval_string);
 		efree(new_value);
@@ -1954,7 +1550,6 @@ DBGP_FUNC(property_value)
 	/* Set the symbol table corresponding with the requested stack depth */
 	if (context_nr == 0) { /* locals */
 		if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 			function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
 
 			if (depth > 0) {
@@ -1962,11 +1557,7 @@ DBGP_FUNC(property_value)
 			} else {
 				XG(active_execute_data) = EG(current_execute_data);
 			}
-#else
-			XG(active_execute_data) = fse->execute_data;
-#endif
 			XG(active_symbol_table) = fse->symbol_table;
-			XG(active_op_array)     = fse->op_array;
 			XG(This)                = fse->This;
 			XG(active_fse)          = fse;
 		} else {
@@ -1993,10 +1584,9 @@ DBGP_FUNC(property_value)
 	}
 	if (add_variable_contents_node(*retval, CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1, 1, 0, 0, options TSRMLS_CC) == FAILURE) {
 		options->max_data = old_max_data;
-		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTANT);
+		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTENT);
 	}
 	options->max_data = old_max_data;
-	XG(active_op_array) = NULL;
 }
 
 static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, void *options)
@@ -2014,12 +1604,16 @@ static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, vo
 	}
 }
 
-static int xdebug_add_filtered_symboltable_var(zval *symbol XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+static int xdebug_add_filtered_symboltable_var(zval *symbol TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
 	xdebug_hash *tmp_hash;
 
 	tmp_hash = va_arg(args, xdebug_hash *);
 
+	/* We really ought to deal properly with non-associate keys for symbol
+	 * tables, but for now, we'll just ignore them. */
+	if (!hash_key->arKey || hash_key->nKeyLength == 0) { return 0; }
+
 	if (strcmp("argc", hash_key->arKey) == 0) { return 0; }
 	if (strcmp("argv", hash_key->arKey) == 0) { return 0; }
 	if (hash_key->arKey[0] == '_') {
@@ -2054,8 +1648,8 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 	function_stack_entry *fse;
 	char                 *var_name;
 
-	/* right now, we only have zero or one, one being globals, which is
-	 * always the head of the stack */
+	/* right now, we only have zero, one, or two with one being globals, which
+	 * is always the head of the stack */
 	if (context_id == 1) {
 		/* add super globals */
 		XG(active_symbol_table) = &EG(symbol_table);
@@ -2073,9 +1667,33 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 		return 0;
 	}
 
+	/* add user defined constants */
+	if (context_id == 2) {
+		HashPosition   pos;
+		zend_constant *val;
+
+		zend_hash_internal_pointer_reset_ex(EG(zend_constants), &pos);
+		while (zend_hash_get_current_data_ex(EG(zend_constants), (void **) &val, &pos) != FAILURE) {
+			if (!val->name) {
+				/* skip special constants */
+				goto next_constant;
+			}
+
+			if (val->module_number != PHP_USER_CONSTANT) {
+				/* we're only interested in user defined constants */
+				goto next_constant;
+			}
+
+			add_constant_node(node, val->name, val->name_len, &(val->value), options TSRMLS_CC);
+next_constant:
+			zend_hash_move_forward_ex(EG(zend_constants), &pos);
+		}
+
+		return 0;
+	}
+
 	/* Here the context_id is 0 */
 	if ((fse = xdebug_get_stack_frame(depth TSRMLS_CC))) {
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 		function_stack_entry *old_fse = xdebug_get_stack_frame(depth - 1 TSRMLS_CC);
 
 		if (depth > 0) {
@@ -2083,11 +1701,7 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 		} else {
 			XG(active_execute_data) = EG(current_execute_data);
 		}
-#else
-		XG(active_execute_data) = fse->execute_data;
-#endif
 		XG(active_symbol_table) = fse->symbol_table;
-		XG(active_op_array)     = fse->op_array;
 		XG(This)                = fse->This;
 
 		/* Only show vars when they are scanned */
@@ -2100,7 +1714,7 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 			/* Check for dynamically defined variables, but make sure we don't already
 			 * have them. Also blacklist superglobals and argv/argc */
 			if (XG(active_symbol_table)) {
-				zend_hash_apply_with_arguments(XG(active_symbol_table) XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) xdebug_add_filtered_symboltable_var, 1, tmp_hash);
+				zend_hash_apply_with_arguments(XG(active_symbol_table) TSRMLS_CC, (apply_func_args_t) xdebug_add_filtered_symboltable_var, 1, tmp_hash);
 			}
 
 			/* Add all the found variables to the node */
@@ -2125,7 +1739,6 @@ static int attach_context_vars(xdebug_xml_node *node, xdebug_var_export_options
 
 		XG(active_symbol_table) = NULL;
 		XG(active_execute_data) = NULL;
-		XG(active_op_array)     = NULL;
 		XG(This)                = NULL;
 		return 0;
 	}
@@ -2178,10 +1791,16 @@ DBGP_FUNC(context_names)
 	xdebug_xml_add_attribute(child, "name", "Locals");
 	xdebug_xml_add_attribute(child, "id", "0");
 	xdebug_xml_add_child(*retval, child);
+
 	child = xdebug_xml_node_init("context");
 	xdebug_xml_add_attribute(child, "name", "Superglobals");
 	xdebug_xml_add_attribute(child, "id", "1");
 	xdebug_xml_add_child(*retval, child);
+
+	child = xdebug_xml_node_init("context");
+	xdebug_xml_add_attribute(child, "name", "User defined constants");
+	xdebug_xml_add_attribute(child, "id", "2");
+	xdebug_xml_add_child(*retval, child);
 }
 
 DBGP_FUNC(context_get)
@@ -2561,7 +2180,7 @@ int xdebug_dbgp_init(xdebug_con *context, int mode)
 	xdebug_xml_add_text(child, xdstrdup(XDEBUG_COPYRIGHT));
 	xdebug_xml_add_child(response, child);
 
-	if (strcmp(context->program_name, "-") == 0) {
+	if (strcmp(context->program_name, "-") == 0 || strcmp(context->program_name, "Command line code") == 0) {
 		xdebug_xml_add_attribute_ex(response, "fileuri", xdstrdup("dbgp://stdin"), 0, 1);
 	} else {
 		xdebug_xml_add_attribute_ex(response, "fileuri", xdebug_path_to_url(context->program_name TSRMLS_CC), 0, 1);
@@ -2723,7 +2342,7 @@ int xdebug_dbgp_error(xdebug_con *context, int type, char *exception_type, char
 	return 1;
 }
 
-int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, char *file, long lineno, int type, char *exception, char *message)
+int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, char *file, long lineno, int type, char *exception, int code, char *message)
 {
 	xdebug_xml_node *response, *error_container;
 	TSRMLS_FETCH();
@@ -2759,6 +2378,9 @@ int xdebug_dbgp_breakpoint(xdebug_con *context, xdebug_llist *stack, char *file,
 	if (exception) {
 		xdebug_xml_add_attribute_ex(error_container, "exception", xdstrdup(exception), 0, 1);
 	}
+	if (code) {
+		xdebug_xml_add_attribute_ex(error_container, "code", xdebug_sprintf("%lu", code), 0, 1);
+	}
 	if (message) {
 		xdebug_xml_add_text(error_container, xdstrdup(message));
 	}
